/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 * 
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.1.1.1 $
 * $Header: /home/cvspublic/opendj/opendj-chat-client/src/org/opendj/chat/ChatApplet.java,v 1.1.1.1 2001/09/29 20:20:20 jonathan Exp $
 * $Log: ChatApplet.java,v $
 * Revision 1.1.1.1  2001/09/29 20:20:20  jonathan
 * Imported opendj-chat-client
 *
 */
package org.opendj.chat;

import java.awt.Toolkit;
import java.awt.Frame;
import java.awt.Window;
import java.awt.Panel;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Rectangle;
import java.awt.Dimension;
import java.awt.TextComponent;
import java.awt.TextArea;
import java.awt.TextField;
import java.awt.Button;
import java.awt.List;
import java.awt.Label;
import java.awt.Event;
import java.applet.Applet;
import java.util.Properties;
import java.util.Vector;

public class ChatApplet extends Applet implements ChatSocketListener {
    
    // AWT Components
    private TextField itsTextField;
    private TextArea itsTextArea;
    private Button itsLogoutButton, itsShowUsersButton;
    private List itsUserList;
    private Label itsLoggedInLabel;

    // Used to manage placement of child windows (for private conversations)
    private Rectangle itsLastPlacement = null;
    private int SCREENHEIGHT = 0;
    private int SCREENWIDTH  = 0;
    private Vector itsChildFrames = null;

    // CONSTANTS
    public static String NEWLINE = "\n";
    public static final String LABEL_NOT_LOGGED_IN = "Not Logged In         ";
    public static final int DEFAULT_PORT = 8029;
    public static final String DEFAULT_URLFRAMENAME = "showURL";
    // public static final Color MAIN_BGCOLOR = new Color(0xf1f4ff);
    public static final Color MAIN_BGCOLOR = new Color(0xffffff);
    public static final Color USER_BGCOLOR = new Color(0xffffff);
    public static final Font VARIABLE_FONT = new Font("Arial",Font.PLAIN,12);
    public static final Font FIXED_FONT = new Font("Courier",Font.PLAIN,12);
    public static final int TEXTAREA_WRAPSIZE = 75;
    public static final String MAIN_SOCKLISTENER_NAME = "------PUBLIC------";

    // Threads
    private Thread itsChatSocketThread = null;
    
    // ChatSocket
    private ChatSocket itsChatSocket = null;

    // State variables
    public static final int STATE_INIT  = 0;
    public static final int STATE_NAMED = 1;
    public static final int STATE_DONE  = 2;

    private int itsState = STATE_INIT;
    private String itsUserName = null;

    private Repainter itsRepainter = null;

    // Variables used when this is running inside the encoder/client
    Properties itsProps = null;

    // Applet parameters
    String HOSTNAME = null;
    String ROOMNAME = null;
    int PORT = -1;

    // Only used when a DJ has already authenticated with our server
    String DJEMAIL   = null;
    String EPASSWORD = null;

    public ChatApplet () {
        itsProps = null;
    }

    public void setProps ( Properties theProps ) {
        itsProps = theProps;
    }

    public void init() {
        
        loadParameters();

        // Init misc vars
        itsChildFrames = new Vector();

        // Define various AWT components we'll use
        itsTextField = new TextField(40);
        itsTextArea = new TextArea(10, 30);
        itsLogoutButton = new Button("Leave the Chat");
        itsUserList = new List(10, false);
        itsShowUsersButton = new Button("Refresh User List");
        itsLoggedInLabel = new Label(LABEL_NOT_LOGGED_IN);

        // Set various properties on these components
        itsTextArea.setEditable(false);
        itsTextArea.setFont(FIXED_FONT);
        itsTextArea.setBackground(MAIN_BGCOLOR);
        itsTextField.setFont(VARIABLE_FONT);
        itsTextField.setEnabled(false);
        itsTextField.setBackground(USER_BGCOLOR);
        itsUserList.setFont(FIXED_FONT);
        itsUserList.setBackground(USER_BGCOLOR);

        // Set the main layout
        setLayout(new BorderLayout());

        // We'll have a top panel for the message text field
        Panel aTopPanel = new Panel();
        Panel aRealTopPanel = new Panel();
        aTopPanel.setLayout(new BorderLayout());
        aRealTopPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
        aRealTopPanel.add(new Label("Message:"));
        aRealTopPanel.add(itsTextField);
        aRealTopPanel.add(itsLoggedInLabel);
        aTopPanel.add("Center", aRealTopPanel);
        // Just to push up the bottom so it doesn't clip on Netscape
        aTopPanel.add("South", new Label("     "));

        // We'll have a right panel for the user list and various buttons
        Panel aRightPanel = new Panel();
        aRightPanel.setLayout(new BorderLayout());
        aRightPanel.add("North", new Label("Current Users:"));
        aRightPanel.add("Center", itsUserList);
        Panel aRightBottomPanel = new Panel();
        aRightBottomPanel.setLayout(new BorderLayout());
        aRightBottomPanel.add("North", itsShowUsersButton);
        aRightBottomPanel.add("South", itsLogoutButton);
        aRightPanel.add("South", aRightBottomPanel);

        // Add the panels and the main text area to the main layout
        add("South", aTopPanel);
        add("East", aRightPanel);
        add("Center", itsTextArea);

        setBackground(USER_BGCOLOR);
    }
    
    public void start() {
        try {
            debug("Pre-Start...");

            // Setup socket
            itsChatSocket = new ChatSocket(this, HOSTNAME, PORT, 
                                           ROOMNAME, DJEMAIL, EPASSWORD);
            itsChatSocket.addListener(this);
            itsChatSocket.connect();

            debug("Start...");

            // Start active socket object...
            itsChatSocketThread = new Thread(itsChatSocket);
            itsChatSocketThread.start();

            // refreshUserList();
            
            itsRepainter = new Repainter(this);
            Thread aRepaintThread = new Thread(itsRepainter);
            aRepaintThread.start();

            if ( DJEMAIL == null ) {
                pickNewName();
            }

        } catch (Exception e) {
            debug("ERROR!", e);
        }
    }
    
    public void stop () {
        debug("Stop...");
        endChatSession();
    }

    private void endChatSession() {
        killAllChildFrames();
        itsChatSocket.shutdown();
        itsRepainter.finish();
    }

    private void killAllChildFrames() {
        if ( itsChildFrames != null ) {
            int aNumChildren = itsChildFrames.size();
            for ( int i=0; i<aNumChildren; i++ ) {
                ((PrivateChatFrame) itsChildFrames.elementAt(i)).disconnect();
            }
        }
    }
    
    private void sendToPrivateChat( String theRemoteUser, String theMessage ) {

        System.err.print("Sending to child: " + theRemoteUser + "...");

        // Ignore it if we don't have a name yet
        if ( itsState != STATE_NAMED ) return;

        // This returns true if a visible chat is found
        PrivateChatFrame aChatFrame = removeInvisibleChats(theRemoteUser);
        if ( aChatFrame != null ) {
            if ( theMessage != null ) {
                debug("visible child found, sending message.");
                aChatFrame.messageArrived(theMessage);
            }
            ((Window) aChatFrame).toFront();
            return;
        }

        aChatFrame = new PrivateChatFrame(this, 
                                          theRemoteUser,
                                          theMessage);
        itsChildFrames.addElement(aChatFrame);
        debug("added new child.");
    }

    public synchronized void removePrivateChat ( PrivateChatFrame theChild ) {
        debug("Removing child: " + theChild.getRemoteUser());
        if ( itsChildFrames != null ) {
            int aNumChildren = itsChildFrames.size();
            for ( int i=0; i<aNumChildren; i++ ) {
                if ( itsChildFrames.elementAt(i) == theChild ) {
                    itsChildFrames.removeElementAt(i);
                    return;
                }
            }
        }
    }

    public synchronized PrivateChatFrame removeInvisibleChats (String theRemoteUser) {
        PrivateChatFrame aVisibleChatFound = null;
        if ( itsChildFrames != null ) {
            int aNumChildren = itsChildFrames.size();
            PrivateChatFrame aChild = null;
            for ( int i=0; i<aNumChildren; i++ ) {
                aChild = (PrivateChatFrame) itsChildFrames.elementAt(i);
                // Same name, different identity is a twin...
                if ( aChild.getRemoteUser().equals(theRemoteUser) ) {
                    if ( !aChild.isVisible()  ) {
                        removePrivateChat(aChild);
                        aChild.disconnect();

                    } else if ( aVisibleChatFound != null ) {
                        debug("ERROR: Multiple visible chats for "
                                           + "the same remote user found "
                                           + ", removing...");
                        removePrivateChat(aChild);
                        aChild.disconnect();

                    } else {
                        aVisibleChatFound = aChild;
                    }
                }
            }
        }
        return aVisibleChatFound;
    }

    private void addToUserList( String theUserName ) {

        if ( theUserName == null ) return;

        int i = 0;
        int aListSize = itsUserList.countItems();

        for ( i=0; i<aListSize; i++ ) {
            // Ignore duplicates
            if ( theUserName.equals(itsUserList.getItem(i)) ) return;
        }

        String aLowerCaseUserName = theUserName.toLowerCase();
        for ( i=0; i<aListSize; i++ ) {
            // We've found the right place to insert...
            if ( aLowerCaseUserName.compareTo(itsUserList.getItem(i).toLowerCase()) > 0 ) {
                break;
            }
        }

        itsUserList.addItem(theUserName, i);
    }

    private boolean userExists ( String theUserName ) {
        if ( theUserName == null || theUserName.length() == 0 ) return false;

        // First check existing child frames
        int i=0;
        int stop = itsChildFrames.size();
        if ( itsChildFrames != null ) {
            for ( i=0; i<stop; i++ ) {
                if ( ((PrivateChatFrame) itsChildFrames.elementAt(i)).getRemoteUser().equals(theUserName) ) {
                    return true;
                }
            }
        }

        // Now check user list
        stop = itsUserList.countItems();
        for ( i=0; i<stop; i++ ) {
            if ( (itsUserList.getItem(i)).equals(theUserName) ) {
                return true;
            }
        }
        return false;
    }

    private void removeFromUserList( String theUserName ) {

        if ( theUserName == null ) return;

        int i = 0;
        int aListSize = itsUserList.countItems();

        for ( i=0; i<aListSize; i++ ) {
            if ( theUserName.equals(itsUserList.getItem(i)) ) {
                itsUserList.delItem(i);
                return;
            }
        }
    }

    private void outputAllChildren() {
        String rstr = "Children:\n";
        if ( itsChildFrames != null ) {
            int aNumChildren = itsChildFrames.size();
            for ( int i=0; i<aNumChildren; i++ ) {
                rstr += ((PrivateChatFrame) itsChildFrames.elementAt(i)).getName()
                    + "\n";
            }
        }
        rstr += "END OF CHILD LIST";
        debug(rstr);
    }
    
    public boolean action(Event e, Object arg) {
        
        try {
            if (e.target == itsTextField) {
                String aMessageText = itsTextField.getText();
                if ( aMessageText != null ) {
                    aMessageText = aMessageText.trim();
                    if ( aMessageText.length() > 0 ) {
                        ((TextComponent) itsTextField).setText("");
                        itsChatSocket.sendData(ChatSocketCommands.REQ_MESSAGE, 
                                               aMessageText);
                    }
                }
                return true;
            }
            
            if (e.target == itsUserList) {

                String aUserName = itsUserList.getSelectedItem();

                // If we clicked on ourselves?
                if ( !aUserName.equals(itsUserName) ) {
                    sendToPrivateChat(itsUserList.getSelectedItem(), null);
                }
                return true;
            }

            if (e.target == itsLogoutButton) {
                logout();
                return true;            
            }
            
            if (e.target == itsShowUsersButton ) {
                refreshUserList();
                return true;
            }
            
        } catch ( Exception anException ) {
            // uh oh...
            debug("action caused exception: " + anException.toString());
            anException.printStackTrace();
        }
        
        return false;
    }
    
    // Inserts newline chars as necessary so that the text wraps by wrapAmount
    public static String forceTextAreaWrap(String txt, TextArea theTextArea) {
        
        String rstr = "";

        // Calculate wrapamount
        FontMetrics fm = theTextArea.getFontMetrics(theTextArea.getFont());
        Rectangle aTextAreaRect = theTextArea.bounds();
        int wrapAmount = (aTextAreaRect.width / fm.charWidth('W')) - 5;

        // Sanity check -- we won't make things smaller than 25 chars wide
        if ( wrapAmount < 25 ) wrapAmount = 25;

        int wrapHere = wrapAmount;
        int lineStart = 0;
        while ( wrapHere < txt.length() ) {
            
            String substr = new String( txt.substring(lineStart,wrapHere) );
            int lastNewLine = substr.lastIndexOf(NEWLINE);
            int lastSpace = substr.lastIndexOf(' ');
            
            // if there's already a newline char then leave it alone and continue!
            if ( lastNewLine > 0 ) {
                wrapHere = lineStart + lastNewLine + 1;
                rstr += new String(substr.substring(0,lastNewLine));
                
                // otherwise find the last space and replace it with a newline char
            } else {
                if ( lastSpace > 0 ) {     
                    wrapHere = lineStart + lastSpace; 
                    rstr += new String(substr.substring(0,lastSpace)) + "\n>> ";
                } else {
                    // punt
                    rstr += new String(substr) + "\n>> ";
                }
            } 
            lineStart = wrapHere + 1;               
            wrapHere += wrapAmount;
        }
        
        rstr += new String(txt.substring(lineStart));
        
        return rstr;
    }
    
    private void loadParameters () {
        
        String aPortString = null;

        // Are we running inside the encoder/client?
        if ( itsProps != null ) {
            HOSTNAME  = itsProps.getProperty("HOSTNAME");
            ROOMNAME  = itsProps.getProperty("ROOMNAME");
            DJEMAIL   = itsProps.getProperty("DJEMAIL");
            EPASSWORD = itsProps.getProperty("EPASSWORD");
            aPortString = itsProps.getProperty("PORT");
        } else {
            // These parameters are required
            HOSTNAME  = getParameter("HOSTNAME");
            ROOMNAME  = getParameter("ROOMNAME");
            DJEMAIL   = getParameter("DJEMAIL");
            EPASSWORD = getParameter("EPASSWORD");
            // Check if port was a parameter, otherwise use default port
            aPortString = getParameter("PORT");
        }
        try {
            PORT = Integer.parseInt(aPortString);
        } catch ( Exception e ) {
            PORT = DEFAULT_PORT;
        }

        // Grab screen dimensions
        Toolkit aWindowTK = Toolkit.getDefaultToolkit();
        Dimension aScreenDimension = aWindowTK.getScreenSize();
        SCREENHEIGHT = aScreenDimension.height;
        SCREENWIDTH  = aScreenDimension.width;

        // Grab the newline constant
        ChatApplet.NEWLINE = System.getProperty("line.separator");
    }

    public String getLocalUser () { return itsUserName; }
    public ChatSocket getChatSocket () { return itsChatSocket; }

    public synchronized Rectangle registerNewChild(PrivateChatFrame theChildFrame,
                                                   Dimension theSize) {
        Rectangle aRect = bounds();
        int anXPos = aRect.x + aRect.width;

        if ( anXPos + theSize.width > SCREENWIDTH ) {
            anXPos = SCREENWIDTH - theSize.width;
        }

        int aYPos = aRect.y;
        if ( itsLastPlacement != null ) {
            if ( itsLastPlacement.y + itsLastPlacement.height < SCREENHEIGHT ) {
                aYPos = itsLastPlacement.y + itsLastPlacement.height;
            } else {
                aYPos = SCREENHEIGHT - theSize.height;
            }
        }

        itsLastPlacement = new Rectangle( anXPos, 
                                          aYPos, 
                                          theSize.width, 
                                          theSize.height );

        return itsLastPlacement;
    }

    // ChatSocketListener interface methods
    public String getName() { return MAIN_SOCKLISTENER_NAME; }

    // Main applet is the last to handle commands
    public int getPriority() { return Integer.MIN_VALUE; }

    public boolean handleServerCommand( String theCommand, 
                                        String theData,
                                        String theSource ) {
        boolean wasHandled = false;

        if (theCommand.equals(ChatSocketCommands.RES_MESSAGE)) {
            itsTextArea.appendText
                (ChatApplet.forceTextAreaWrap("<" + theSource + "> " + theData, itsTextArea) + "\n");
            wasHandled = true;

        } else if ( theCommand.equals(ChatSocketCommands.RES_PRIVATEMESSAGE) ) {
            // If we got this message with a source, that means
            // that no other PrivateChatFrame handled it, (because the parent
            // has the lowest priority, so we'll popup a new PrivateChatFrame
            
            // First do a sanity check to make sure this is a real user
            if ( !userExists(theSource) ) {
                // Nonexistant user -- consume the event
                debug("Consuming event because userExists("+theSource+") returned false");
                wasHandled = true;

            } else if ( !theSource.equals(itsUserName) ) {
                // Cannot send private messages to self!
                sendToPrivateChat(theSource, theData);
                wasHandled = true;
            }

        } else if (theCommand.equals(ChatSocketCommands.RES_JOIN)) {
            // A new user has joined the chat...
            addToUserList(theData);
            itsTextArea.appendText(theData + " joined the chat.\n");
            wasHandled = true;

        } else if (theCommand.equals(ChatSocketCommands.RES_LISTUSER)) {
            if ( !userExists(theData) ) {
                addToUserList(theData);
            }
            wasHandled = true;

        } else if (theCommand.equals(ChatSocketCommands.RES_LEAVEUSER)) {
            // Someone has left the chat...
            removeFromUserList(theData);
            itsTextArea.appendText(theData + " left the chat.\n");
            wasHandled = true;

        } else if (theCommand.equals(ChatSocketCommands.RES_LEAVETIMEOUT)) {
            // Someone has left the chat...
            removeFromUserList(theData);
            itsTextArea.appendText(theData + " left the chat because of a connection timeout.\n");
            wasHandled = true;

        } else if (theCommand.equals(ChatSocketCommands.RES_LEAVEBOOTED)) {
            // Someone has left the chat...
            removeFromUserList(theData);
            itsTextArea.appendText(theData + " was booted from the chat.\n");
            wasHandled = true;

        } else if (theCommand.equals(ChatSocketCommands.RES_ROOMFULL)) {
            // What to do here?
            itsTextArea.appendText("This chat room is full.  You can't enter right now.\n");
            wasHandled = true;
        }

        return wasHandled;
    }

    private void pickNewName() {
        PickNameDialog aPNDialog = new PickNameDialog(this);
    }

    public void setNewName(String theUserName) {
        itsUserName = theUserName;

        itsLoggedInLabel.setText("Logged in as: " + itsUserName);
        itsTextField.setEnabled(true);
        refreshUserList();

        itsState = STATE_NAMED;
        repaint();
    }

    public void logout () {
        itsLoggedInLabel.setText(LABEL_NOT_LOGGED_IN);
        itsTextArea.appendText("Leaving chat....\n");
        itsTextField.setEnabled(false);
        itsChatSocket.shutdown();
        removeFromUserList(itsUserName);
        itsState = STATE_DONE;
    }

    public int getState() { return itsState; }

    public void refreshUserList () {
        itsChatSocket.sendData(ChatSocketCommands.REQ_LISTUSERS);
        repaint();
    }

    public void debug ( Object o ) {
        if ( o == null ) o = new String("null");
        System.err.println("[applet] " + o.toString());
    }

    public void debug ( Object o, Exception e ) {
        if ( o == null ) o = new String("null");
        System.err.println("[applet] " + o.toString());
        e.printStackTrace();
    }

    /**
     * Borrowed from Util.java
     */
    public static String stringReplace ( String src, String find, String replace ) {

        if ( src              == null || 
             find             == null ||
             replace          == null || 
             src.length()     == 0    ||
             find.length()    == 0    ||
             replace.length() == 0 ) {

            return src;
        }

        String rstr = "";
        int idx = src.indexOf(find);
        int startIdx = 0;
        while ( idx != -1 ) {
            rstr += src.substring(startIdx, idx) + replace;
            startIdx = idx + find.length();
            idx = src.indexOf(find, startIdx);
        }
        rstr += src.substring(startIdx, src.length());

        return rstr;        
    }

    // Polled by JavaScript to find out when to close the window
    public boolean isRunning () {
        debug("ChatApplet.isRunning called.");
        return ( itsState != STATE_DONE );
    }
}
