/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 * 
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.1.1.1 $
 * $Header: /home/cvspublic/opendj/opendj-chat-client/src/org/opendj/chat/PickDJNameDialog.java,v 1.1.1.1 2001/09/29 20:20:20 jonathan Exp $
 * $Log: PickDJNameDialog.java,v $
 * Revision 1.1.1.1  2001/09/29 20:20:20  jonathan
 * Imported opendj-chat-client
 *
 */
package org.opendj.chat;

import java.awt.Dialog;
import java.awt.Window;
import java.awt.Frame;
import java.awt.Panel;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.TextField;
import java.awt.Button;
import java.awt.Label;
import java.awt.Event;
import java.awt.Color;

public class PickDJNameDialog extends Dialog implements ChatSocketListener {

    public static final String SOCKLISTENER_NAME = "--PickDJNameDialog--";

    private ChatApplet itsApp        = null;
    private ChatSocket itsChatSocket = null;

    private Label     itsMessageLabel  = null;
    private TextField itsEmailField    = null;
    private TextField itsPasswordField = null;
    private Button    itsOKButton      = null;
    private Button    itsCancelButton  = null;

    private String itsEmail = null;
    private String itsPassword = null;

    public PickDJNameDialog(ChatApplet theApp) {
        super(new Frame(), "Login to Chat as a DJ", true);

        itsApp = theApp;
        itsChatSocket = theApp.getChatSocket();
        itsChatSocket.addListener(this);
        setLayout(new BorderLayout());

        itsEmailField = new TextField(40);
        itsEmailField.setFont(ChatApplet.VARIABLE_FONT);
        itsPasswordField = new TextField(40);
        itsPasswordField.setFont(ChatApplet.VARIABLE_FONT);
        itsPasswordField.setEchoCharacter('*');

        itsOKButton        = new Button("OK");
        itsCancelButton    = new Button("Cancel");
        itsMessageLabel    = new Label("Enter your DJ email and password");

        Panel aButtonPanel = new Panel();
        aButtonPanel.setLayout(new BorderLayout());
        aButtonPanel.add("West", itsOKButton);
        aButtonPanel.add("East", itsCancelButton);

        Panel aFieldPanel = new Panel();
        aFieldPanel.setLayout(new BorderLayout());
        aFieldPanel.add("North", itsEmailField);
        aFieldPanel.add("South", itsPasswordField);

        Panel aLabelPanel = new Panel();
        aLabelPanel.setLayout(new BorderLayout());
        aLabelPanel.add("North", new Label("Email:"));
        aLabelPanel.add("South", new Label("Password:"));

        Panel aGroupedPanel = new Panel();
        aGroupedPanel.setLayout(new BorderLayout());
        aGroupedPanel.add("West", aLabelPanel);
        aGroupedPanel.add("East", aFieldPanel);

        add("North",  itsMessageLabel);
        add("Center", aGroupedPanel);
        add("South",  aButtonPanel);

        setResizable(false);

        ((Window) this).pack();
        ((Window) this).show();
    }

    public boolean action(Event e, Object arg) {
        if ( e.target == itsOKButton || 
             e.target == itsPasswordField ) {

            itsEmail = itsEmailField.getText();
            itsPassword = itsPasswordField.getText();

            if ( itsEmail != null && itsEmail.length() > 0 &&
                 itsPassword != null && itsPassword.length() > 0 ) {
                itsChatSocket.sendData( ChatSocketCommands.REQ_JOINASDJ, itsEmail, itsPassword );
            }
            return true;

        } else if ( e.target == itsCancelButton ) {
            dispose();
            itsApp.logout();
            return true;
        }
        return false;
    }

    public String getName() { return SOCKLISTENER_NAME; }
    public int getPriority() { return 0; }

    public boolean handleServerCommand ( String theCommand,
                                         String theData,
                                         String theSource ) {
        boolean wasHandled = false;
        if ( theCommand.equals(ChatSocketCommands.RES_NAMEOK) ) {
            debug("name ok");
            // Name is OK...
            if ( itsApp.getState() == ChatApplet.STATE_NAMED ) {
                itsChatSocket.reset();
            }
            itsApp.setNewName(theData);
            dispose();
            itsChatSocket.removeListener(this);
            return true;

        } else if (theCommand.equals(ChatSocketCommands.RES_NOSUCHDJ)) {
            // Tell them the name was too short
            itsMessageLabel.setText("Email address was incorrect.");
            itsMessageLabel.setForeground(Color.red);
            debug("no such dj");
            return true;

        } else if (theCommand.equals(ChatSocketCommands.RES_BADPASSWORD)) {
            // Tell them the name was too long
            itsMessageLabel.setText("The password was incorrect.");
            itsMessageLabel.setForeground(Color.red);
            debug("bad password");
            return true;

        } else if (theCommand.equals(ChatSocketCommands.RES_NAMETAKEN)) {
            // Tell them the name was taken
            itsMessageLabel.setText("The name you chose is already in use.");
            itsMessageLabel.setForeground(Color.red);
            debug("name taken");
            return true;
        }
        return false;
    }

    private void debug ( Object o ) {
        if ( o == null ) o = new String("null");
        itsApp.debug("[PickDJNameDialog] " + o);
    }
}
