/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 * 
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.1.1.1 $
 * $Header: /home/cvspublic/opendj/opendj-chat-client/src/org/opendj/chat/PickNameDialog.java,v 1.1.1.1 2001/09/29 20:20:20 jonathan Exp $
 * $Log: PickNameDialog.java,v $
 * Revision 1.1.1.1  2001/09/29 20:20:20  jonathan
 * Imported opendj-chat-client
 *
 */
package org.opendj.chat;

import java.awt.Dialog;
import java.awt.Window;
import java.awt.Frame;
import java.awt.Panel;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.TextField;
import java.awt.Button;
import java.awt.Label;
import java.awt.Event;
import java.awt.Color;

public class PickNameDialog extends Dialog implements ChatSocketListener {

    public static final String SOCKLISTENER_NAME = "--PickNameDialog--";

    private ChatApplet itsApp        = null;
    private ChatSocket itsChatSocket = null;

    private Label itsMessageLabel = null;
    private TextField itsUserNameField = null;
    private Button itsOKButton = null;
    private Button itsCancelButton = null;
    private Button itsLoginAsDJButton = null;

    private String itsUserName = null;

    public PickNameDialog(ChatApplet theApp) {
        super(new Frame(), "Choose a chat name", true);

        itsApp = theApp;
        itsChatSocket = theApp.getChatSocket();
        itsChatSocket.addListener(this);
        setLayout(new BorderLayout());

        itsUserNameField = new TextField(40);
        itsUserNameField.setFont(ChatApplet.VARIABLE_FONT);

        itsOKButton        = new Button("OK");
        itsCancelButton    = new Button("Cancel");
        itsLoginAsDJButton = new Button("Login As A DJ");
        itsMessageLabel    = new Label("Please choose a new name");

        Panel aButtonPanel = new Panel();
        aButtonPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
        aButtonPanel.add(itsUserNameField);
        aButtonPanel.add(itsOKButton);
        aButtonPanel.add(itsCancelButton);

        Panel aDJPanel = new Panel();
        aDJPanel.add("West", new Label("To login as a DJ, click this button"));
        aDJPanel.add("East", itsLoginAsDJButton);

        add("North", itsMessageLabel);
        add("Center", aButtonPanel);
        add("South", aDJPanel);

        setResizable(false);

        ((Window) this).pack();
        ((Window) this).show();
    }

    public boolean action(Event e, Object arg) {
        if ( e.target == itsOKButton || e.target == itsUserNameField ) {
            itsUserName = itsUserNameField.getText();
            if ( itsUserName != null && itsUserName.length() > 0 ) {
                itsChatSocket.sendData( ChatSocketCommands.REQ_JOIN, itsUserName );
            }
            return true;

        } else if ( e.target == itsCancelButton ) {
            itsChatSocket.removeListener(this);
            dispose();
            itsApp.logout();
            return true;

        } else if ( e.target == itsLoginAsDJButton ) {
            itsChatSocket.removeListener(this);
            dispose();
            PickDJNameDialog p = new PickDJNameDialog(itsApp);
            return true;
        }
        return false;
    }

    public String getName() { return SOCKLISTENER_NAME; }
    public int getPriority() { return 0; }

    public boolean handleServerCommand ( String theCommand,
                                         String theData,
                                         String theSource ) {
        boolean wasHandled = false;
        if ( theCommand.equals(ChatSocketCommands.RES_NAMEOK) ) {
            debug("name ok");
            // Name is OK...
            if ( itsApp.getState() == ChatApplet.STATE_NAMED ) {
                itsChatSocket.reset();
            }
            itsApp.setNewName(itsUserName);
            dispose();
            return true;

        } else if (theCommand.equals(ChatSocketCommands.RES_NAMETOOSHORT)) {
            // Tell them the name was too short
            itsMessageLabel.setText("The name you chose was too short.");
            itsMessageLabel.setForeground(Color.red);
            debug("name too short");
            return true;

        } else if (theCommand.equals(ChatSocketCommands.RES_NAMETOOLONG)) {
            // Tell them the name was too long
            itsMessageLabel.setText("The name you chose was too long.");
            itsMessageLabel.setForeground(Color.red);
            debug("name too long");
            return true;

        } else if (theCommand.equals(ChatSocketCommands.RES_NAMETAKEN)) {
            // Tell them the name was taken
            itsMessageLabel.setText("The name you chose is already in use.");
            itsMessageLabel.setForeground(Color.red);
            debug("name taken");
            return true;

        } else if (theCommand.equals(ChatSocketCommands.RES_NAMEISDJ)) {
            // Tell them the name was taken
            itsMessageLabel.setText("The name you chose is reserved.");
            itsMessageLabel.setForeground(Color.red);
            debug("name is DJ");
            return true;

        } else if (theCommand.equals(ChatSocketCommands.RES_MUSTBEDJ)) {
            // Tell them the name was too short
            itsMessageLabel.setText("You must be a DJ to chat in this room.");
            itsMessageLabel.setForeground(Color.red);
            debug("must be dj");
            return true;
        }
        return false;
    }

    private void debug ( Object o ) {
        if ( o == null ) o = new String("null");
        itsApp.debug("[PickNameDialog] " + o);
    }
}
