/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 * 
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.1.1.1 $
 * $Header: /home/cvspublic/opendj/opendj-chat-client/src/org/opendj/chat/PrivateChatFrame.java,v 1.1.1.1 2001/09/29 20:20:20 jonathan Exp $
 * $Log: PrivateChatFrame.java,v $
 * Revision 1.1.1.1  2001/09/29 20:20:20  jonathan
 * Imported opendj-chat-client
 *
 */
package org.opendj.chat;

import java.awt.*;

class PrivateChatFrame extends Frame {

    // CONSTANTS
    public static final int HEIGHT = 200;
    public static final int WIDTH  = 350;
    public static final int TEXTAREA_WRAPSIZE = 35;

    // AWT Components
    private TextArea  itsTextArea;
    private TextField itsSendField;
    private Button    itsCloseButton;

    private ChatApplet itsParentApplet = null;
    private String     itsLocalUser    = null;
    private String     itsRemoteUser   = null;

    // For sending data to the server
    private ChatSocket itsChatSocket   = null;

    public PrivateChatFrame( ChatApplet theParent, 
                             String theRemoteUser ) {
        super("Private Conversation with " + theRemoteUser);
        init(theParent, theRemoteUser, null);
    }

    public PrivateChatFrame( ChatApplet theParent, 
                             String theRemoteUser,
                             String theInitialText ) {
        super("Private Conversation with " + theRemoteUser);
        init(theParent, theRemoteUser, theInitialText);
    }

    private void init ( ChatApplet theParent, 
                        String theRemoteUser,
                        String theInitialText ) {
        itsParentApplet = theParent;
        itsChatSocket = theParent.getChatSocket();
        itsLocalUser = theParent.getLocalUser();
        itsRemoteUser = theRemoteUser;
        
        // Do placement
        Rectangle aNewRect
            = itsParentApplet.registerNewChild(this, new Dimension(WIDTH, HEIGHT));
        
        // Standard size
        resize( WIDTH, HEIGHT );
        setResizable(true);

        // Set placement
        move(aNewRect.x, aNewRect.y);
        
        // Define the AWT components
        itsTextArea = new TextArea(10, 30);
        itsTextArea.setEditable(false);
        itsTextArea.setFont(ChatApplet.FIXED_FONT);
        itsTextArea.setBackground(ChatApplet.MAIN_BGCOLOR);
        if ( theInitialText != null ) {
            itsTextArea.appendText
                (ChatApplet.forceTextAreaWrap(theInitialText,
                                              itsTextArea) + "\n");
        }

        itsSendField = new TextField(20);
        itsSendField.setFont(ChatApplet.VARIABLE_FONT);

        itsCloseButton = new Button("Close");

        Panel aBottomPanel = new Panel();
        aBottomPanel.setLayout(new BorderLayout());
        aBottomPanel.add("Center", itsSendField);
        aBottomPanel.add("East", itsCloseButton);

        // Arrange the components
        setLayout( new BorderLayout() );
        add("North", new Label("Private Conversation with " + theRemoteUser));
        add("Center", itsTextArea);
        add("South", aBottomPanel);

        pack();
        show();
    }

    public void finalize () {
        disconnect();
    }

    public String getRemoteUser() { return itsRemoteUser; }

    public void disconnect () {

        System.err.println("PrivateChatFrame.disconnect called:");
        Exception e = new Exception("");
        e.printStackTrace();

        if ( itsChatSocket != null ) {
            itsChatSocket = null;
        }
        if ( itsParentApplet != null ) {
            itsParentApplet.removePrivateChat(this);
            itsParentApplet = null;
        }
        this.dispose();
    }

    public void messageArrived ( String theMessage ) {
        itsTextArea.appendText
            (ChatApplet.forceTextAreaWrap(theMessage,
                                          itsTextArea) + "\n");
    }

    public boolean action(Event e, Object arg) {
        try {
            // Hanlde window destroy first
            if ( e.id == Event.WINDOW_DESTROY ) {
                disconnect();
                return false;
            }

            if ( e.target == itsCloseButton ) {
                disconnect();
                return true;
            }
            if ( e.target == itsSendField ) {

                String aMessageText = itsSendField.getText();
                ((TextComponent) itsSendField).setText("");

                itsChatSocket.sendData(ChatSocketCommands.REQ_MESSAGE, 
                                       aMessageText,
                                       itsRemoteUser);

                itsTextArea.appendText
                    (ChatApplet.forceTextAreaWrap("<" 
                                                  + itsLocalUser
                                                  + "> " 
                                                  + aMessageText,
                                                  itsTextArea) + "\n");
                
                return true;
            }
        } catch ( Exception anException ) {
            // uh oh...
            System.err.println("action caused exception: " + anException.toString());
            anException.printStackTrace();
        }
        return false;
    }
}
